Attribute VB_Name = "modulePortIO"
Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
Declare Sub Out Lib "INPOUT32.DLL" Alias "Out32" (ByVal Addr%, ByVal Value As Integer)
Declare Function Inp Lib "INPOUT32.DLL" Alias "Inp32" (ByVal Addr%) As Integer

'declarations of constants for serial communication
' output register constants first
    Const SDA_ONE = 1              '00000001
    Const SDA_ZERO = 0             '00000000
Public SCL_HIGH As Integer
Public SCL_LOW As Integer
Public FreeI2C As Integer
Public TestPinHigh As Boolean
Public Const I2C_PGA309_Addr As Byte = &H80    ' 7'b1000000
Public Const I2C_EEPROM_Addr As Byte = &HA0    ' 7'b1010000
    Const WriteBit As Byte = 0
    Const ReadBit As Byte = 1
    Const MaxIOAttempts = 100      'max number of attempts to communicate before bailout
    
Public Const ROM_Programmed As Long = &H5449    '0101_0100_0100_1001 -flag word of EEPROM
    
Public Const Base1_Address = &H378 ' LPT 1
Public Const Base2_Address = &H278 ' LPT 2

Public Port_Data As Integer
Public Port_Status As Integer
Public Port_Controls As Integer

Public Sub InitLPTPort()
    Dim InWord As Integer
    On Error Resume Next
    If TestPinHigh Then
        SCL_HIGH = 4 Or 2       '00000110 'also pulls the test pin (#4) high
        SCL_LOW = 4 Or 0         '00000100
    Else
        SCL_HIGH = 2       '00000010 'keeps the test pin (#4) low
        SCL_LOW = 0         '00000000
    End If
    FreeI2C = SDA_ONE Or SCL_HIGH  'default port status - free bus
    Out Port_Data, FreeI2C   'free I2C
            'initialize the port to set the I2C bus high
End Sub

Public Sub I2CStartCond()
  Out Port_Data, (SDA_ONE Or SCL_HIGH)   'free I2C
  Out Port_Data, (SDA_ZERO Or SCL_HIGH)
  Out Port_Data, (SDA_ZERO Or SCL_LOW)
End Sub
Public Sub I2CStopCond()
  Out Port_Data, (SDA_ONE Or SCL_LOW)
  Out Port_Data, (SDA_ZERO Or SCL_LOW)
  Out Port_Data, (SDA_ZERO Or SCL_HIGH)
'  Out Port_Data, (SDA_ONE or SCL_HIGH)
'  Out Port_Data, (SDA_one or SCL_low)
'  Out Port_Data, (SDA_one or SCL_low)    'twice
  Out Port_Data, (SDA_ONE Or SCL_HIGH)   'free I2C
End Sub

Public Sub I2C_Master_Ack()
  Out Port_Data, (SDA_ONE Or SCL_LOW)
  Out Port_Data, (SDA_ZERO Or SCL_LOW)
  Out Port_Data, (SDA_ZERO Or SCL_HIGH)
  Out Port_Data, (SDA_ZERO Or SCL_LOW)
End Sub
Public Sub I2C_No_Ack()
  Out Port_Data, (SDA_ONE Or SCL_LOW)
  Out Port_Data, (SDA_ONE Or SCL_HIGH)
  Out Port_Data, (SDA_ONE Or SCL_LOW)
End Sub

Public Function I2CTransmitByte(SendByte As Byte) As Boolean
Dim j As Integer
Dim Bit As Integer
  For j = 7 To 0 Step -1
    If (SendByte And CByte(2 ^ j)) <> 0 Then
        Bit = SDA_ONE 'if bit = 1
    Else
        Bit = SDA_ZERO
    End If
    Out Port_Data, (Bit Or SCL_LOW)
    Out Port_Data, (Bit Or SCL_HIGH)
    If (((Inp(Port_Status) \ 16) Xor 8) And 1) <> Bit Then
            ' latch the SDA bit to the input and compare with the output
        I2CTransmitByte = False    'return false - the bench is probably not connected
        Exit Function   'bail out
    End If
    Out Port_Data, (Bit Or SCL_LOW)
  Next j
  Out Port_Data, (SDA_ONE Or SCL_LOW)
  Out Port_Data, (SDA_ONE Or SCL_HIGH)
  Bit = ((Inp(Port_Status) \ 16) Xor 8) And 1   ' latch the acknowledgement bit
  If Bit = SDA_ZERO Then   ' if SDA is low  - acknowlwdge recieved
    I2CTransmitByte = True
  Else                     ' if no acknowledgement
    I2CTransmitByte = False
  End If
End Function

Public Function I2CRecieveByte() As Byte
Dim j As Integer
  I2CRecieveByte = 0
  For j = 7 To 0 Step -1
    Out Port_Data, (SDA_ONE Or SCL_LOW)
    Out Port_Data, (SDA_ONE Or SCL_LOW)
    Out Port_Data, (SDA_ONE Or SCL_HIGH)
    I2CRecieveByte = I2CRecieveByte + (2 ^ j) * (((Inp(Port_Status) \ 16) Xor 8) And 1)
  Next j
End Function

Public Function I2CReadWord(DevAddr, RegAddr As Byte) As Long
        ' function reads a 16-bit (2-Byte) word from the I2C device
    Dim CtrlByte, InByte1, InByte2 As Byte
    Dim TryNum As Integer
    Dim Ackn As Boolean
            ' Reading the 2 bytes
    TryNum = 0
    Do
        DoEvents
        Call I2CStartCond
        Ackn = I2CTransmitByte(DevAddr And (Not ReadBit)) 'write the control byte to EEPROM
        If Ackn Then    'acknowledgement recieved
            Ackn = I2CTransmitByte(RegAddr)     'write the addressbyte to EEPROM
        End If
        TryNum = TryNum + 1
    Loop While (Not Ackn) And (TryNum < MaxIOAttempts)    'give it at least MaxIOAttempt tries
    If Not Ackn Then    'if the acknowledge was never obtained
        I2CReadWord = -1  'return -1 and bail out
        Exit Function
    End If
    Call I2CStopCond
    Call I2CStartCond
    Ackn = I2CTransmitByte(DevAddr Or ReadBit) 'write the control byte to EEPROM
    If Ackn Then    'acknowledgement recieved
        InByte1 = I2CRecieveByte()
        Call I2C_Master_Ack
        InByte2 = I2CRecieveByte()
        Call I2C_No_Ack
        Call I2CStopCond
        I2CReadWord = CLng(InByte2) * 256 + CLng(InByte1)
    Else    'if the acknowledge was not obtained
        I2CReadWord = -1  'return -1 and bail out
        Exit Function
    End If
End Function

Public Sub WritePGARegister(RegIndex As Integer)
    If TalkMode = 0 Then  'if the communication mode is I2C
        I2CWritePGARegister (RegIndex) ' read the register
                                'if no valid read - the function I2CWrite....
                                ' will display the message
    Else        'put 1-wire read here
        OW_WritePGARegister (RegIndex)
    End If
End Sub


Public Sub WritePGA()
                ' subroutine to write data into PGA registers
    Dim I As Integer
    Dim Success As Boolean
    formMain.MousePointer = 11 ' change pointer to hourglass
    For I = 1 To 7
        If TalkMode = 0 Then  'if the communication mode is I2C
            Success = I2CWritePGARegister(I)
        Else        '1-wire write here
            Success = OW_WritePGARegister(I)
        End If
        If Not Success Then ' read the register
                        'if no valid read
            Exit For
        End If
    Next I
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub ReadPGARegister(RegIndex As Byte)
    Dim inVal As Long
    If TalkMode = 0 Then  'if the communication mode is I2C
        inVal = I2CReadWord(I2C_PGA309_Addr, RegIndex) ' read the register
    Else        '1-wire read here
        inVal = OW_ReadPGARegister(CInt(RegIndex))
    End If
    If inVal = -1 Then    'if no valid read
        Dim Response
        Response = MsgBox("Error while reading the PGA309 register #" _
                & Str(RegIndex) & ". Read aborted", _
                vbOKOnly + vbCritical, "Error")  'give user the message
    Else
        PGA309Registers(RegIndex) = inVal
        Call UpdatePhysDisplay(False)
    End If
End Sub

Public Sub ReadPGA()
                ' subroutine to write data into PGA registers
    Dim I As Integer
    Dim inVal As Long
    formMain.MousePointer = 11 ' change pointer to hourglass
    For I = 0 To 10
        If TalkMode = 0 Then  'if the communication mode is I2C
            inVal = I2CReadWord(I2C_PGA309_Addr, CByte(I)) ' read the register
        Else        ' 1-wire read here
            inVal = OW_ReadPGARegister(I)
        End If
        If inVal = -1 Then    'if no valid read
            Dim Response
            Response = MsgBox("Error while reading the PGA309 register #" _
                & Str(I) & vbCrtLf & ". Read aborted", _
                vbOKOnly + vbCritical, "Error")  'give user the message
                Exit For
        Else
            PGA309Registers(I) = inVal
        End If
    Next I
    Call UpdatePhysDisplay(False)
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub WriteEEPROM()
    If TalkMode = 0 Then  'if the communication mode is I2C
        Call I2CWriteEEPROM
    Else
        Call OW_WriteEEPROM
    End If
End Sub

Public Sub ReadEEPROM()
        'reads EEPROM content using either I2C or 1-wire routines
    Dim InByte1, InByte2 As Byte
    Dim Ackn As Boolean
    Dim I, CurrAddr, EEPROMDataLength As Integer
    Dim InValue As Long
    Dim Msg, Style, Title, Response
    formMain.MousePointer = 11 ' change pointer to hourglass
            ' Reading the 2 bytes
    If TalkMode = 0 Then
        InValue = I2CReadWord(I2C_EEPROM_Addr, 0)   'reading the first word
    Else
        InValue = OW_ReadEEPROMWord(0)   '1-wire read
    End If
    If InValue = -1 Then
        Msg = "The EEPROM is not responding. Read aborted"  ' Define message.
        Style = vbOKOnly + vbCritical       ' Define buttons.
        Title = "Error"   ' Define title.
        Response = MsgBox(Msg, Style, Title)  'if the table is already full
        formMain.MousePointer = 0
        Exit Sub  ' if user chose No: exit
    ElseIf InValue <> ROM_Programmed Then
        Msg = "The EEPROM doesn't appear to be programmed for PGA309." & vbCrtLf _
            & "Do you still want to load the contents into the memory?"   ' Define message.
        Style = vbYesNo + vbQuestion + vbDefaultButton2   ' Define buttons.
        Title = "Warning"   ' Define title.
        Response = MsgBox(Msg, Style, Title)
        If Response = vbNo Then
            formMain.MousePointer = 0
            Exit Sub  ' if user chose No: exit
        End If
    End If
    For I = 3 To 6      ' read the config registers
        If TalkMode = 0 Then
            PGA309Registers(I) = I2CReadWord(I2C_EEPROM_Addr, 2 * I) 'reading the word
        Else
            PGA309Registers(I) = OW_ReadEEPROMWord(2 * I)  '1-wire read
        End If
    Next I
    If TalkMode = 0 Then
        PGA309Registers(11) = I2CReadWord(I2C_EEPROM_Addr, 14)  'config checksum
    Else
        PGA309Registers(11) = OW_ReadEEPROMWord(14)   'config checksum
    End If
    LookupTableLength = -1
    For I = 0 To MaxLookupTableLength      ' read the lookuptable
        If TalkMode = 0 Then
                 'reading the temp index
            LookupTable(I, 0) = fromTwosCompliment(I2CReadWord(I2C_EEPROM_Addr, 6 * I + 16))
                 'reading the zero val
            LookupTable(I, 1) = fromTwosCompliment(I2CReadWord(I2C_EEPROM_Addr, 6 * I + 18))
                 'reading the gain val
            LookupTable(I, 2) = fromTwosCompliment(I2CReadWord(I2C_EEPROM_Addr, 6 * I + 20))
        Else
                 'reading the temp index
            LookupTable(I, 0) = fromTwosCompliment(OW_ReadEEPROMWord(6 * I + 16))
                 'reading the zero val
            LookupTable(I, 1) = fromTwosCompliment(OW_ReadEEPROMWord(6 * I + 18))
                 'reading the gain val
            LookupTable(I, 2) = fromTwosCompliment(OW_ReadEEPROMWord(6 * I + 20))
        End If
        If LookupTable(I, 0) <> &H7FFF Then 'if it's not the end of the lookup table yet
            LookupTableLength = LookupTableLength + 1
        Else    'if we read the whole thing
            Exit For
        End If
    Next I
    Call UpdateTableDisplay
    Call UpdatePhysDisplay(False)  'update the display w/o changing the
                                'checksum and wrting the registers back
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub I2CWriteEEPROM()
    Dim DataByte As Byte
    Dim Ackn As Boolean
    Dim I, CurrAddr, EEPROMDataLength, TryNum As Integer
    formMain.MousePointer = 11 ' change pointer to hourglass
                    ' create the data sequence for EEPROM
    EEPROMDataLength = CreateDataStream()
        'now start the writing process 8 bytes at a time
    CurrAddr = 0
    TryNum = 0
    Do
        DataByte = I2C_EEPROM_Addr And (Not ReadBit)  'WRITE control word
        Call I2CStartCond
        Ackn = I2CTransmitByte(DataByte) 'write the control byte to EEPROM
        If Ackn Then    'acknowledgement recieved
            DataByte = CurrAddr    'setting the beginning address
            Ackn = I2CTransmitByte(DataByte) 'write the address byte to EEPROM
            If Ackn Then    'acknowledgement recieved
                For I = CurrAddr To CurrAddr + 7 '!!0   'might overstep the array actual length
                                                    'but it should be OK
                    DataByte = EEPROMDataStream(I)    'next byte in sequence
                    Ackn = I2CTransmitByte(DataByte) 'write the data byte to EEPROM
                    If Not Ackn Then Exit For
                Next I
            End If
        End If
        If Ackn Then    'if all the acknowledgements are OK - generate stop
                        ' otherwise repeat the sequience w/o increasing
                        ' the address counter
            Call I2CStopCond
            CurrAddr = CurrAddr + 8 '!!0
            TryNum = 0
        Else
            TryNum = TryNum + 1
        End If
    Loop While ((Not Ackn) Or (CurrAddr <= EEPROMDataLength)) And (TryNum < MaxIOAttempts)
    If Not Ackn Then    ' if the acknowledge was not recieved after MaxIOAttempts
            Dim Response
            Response = MsgBox("The EEPROM is not responding. Write aborted", _
                vbOKOnly + vbCritical, "Error")  'give user the message
    End If
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Function I2CWritePGARegister(RegIndex As Integer) As Boolean
                ' subroutine to write the value into one PGA309 register
                ' returns "True" for success ans "False" for failure to write
    Dim DataByte As Byte
    Dim I, TryNum As Integer
    Dim Reply As Variant
    formMain.MousePointer = 11 ' change pointer to hourglass
    TryNum = 0
    Do
        DataByte = I2C_PGA309_Addr And (Not ReadBit) 'WRITE control word
        Call I2CStartCond
        Ackn = I2CTransmitByte(DataByte) 'write the control byte to PGA309
        If Ackn Then    'acknowledgement recieved
            DataByte = RegIndex    'setting the beginning address
            Ackn = I2CTransmitByte(DataByte) 'write the address byte to PGA309
            If Ackn Then    'acknowledgement recieved
                DataByte = CByte(PGA309Registers(RegIndex) And &HFF)        'LSB
                Ackn = I2CTransmitByte(DataByte) 'write the LSB byte to PGA
                If Ackn Then
                    DataByte = CByte((PGA309Registers(RegIndex) And &HFF00) \ 256)    'MSB
                    Ackn = I2CTransmitByte(DataByte) 'write the MSB byte to PGA
                End If
            End If
        End If
        If Ackn Then    'if all the acknowledgements are OK - generate stop
                        ' otherwise repeat the sequience w/o increasing
                        ' the address counter
            Call I2CStopCond
        Else
            TryNum = TryNum + 1
        End If
    Loop While ((Not Ackn) And (TryNum < MaxIOAttempts))
    If Not Ackn Then    ' if the acknowledge was not recieved after MaxIOAttempts
            Dim Response
            Response = MsgBox("The PGA309 is not responding while writing register #" _
                & Str(RegNum) & vbCrtLf & ". Write aborted", _
                vbOKOnly + vbCritical, "Error")  'give user the message
    End If
    I2CWritePGARegister = Ackn
    formMain.MousePointer = 0 ' change mouse back to default
End Function

Public Sub I2CDetectBench()
            ' the routine checks for the acknowledgements and reads from PGA309 and EEPROM,
    Dim InValPGA, InValEEPROM As Long
    
    Dim Msg, Style, Title, Response
    formMain.MousePointer = 11 ' change pointer to hourglass
            ' Reading the 2 bytes from the PGA309
    InValPGA = I2CReadWord(I2C_PGA309_Addr, 0)   'reading the first word
    If InValPGA = -1 Then
        Msg = "The PGA309 is not responding on I2C bus"  ' Define message.
        Style = vbOKOnly + vbCritical       ' Define buttons.
        Title = "Error"   ' Define title.
        Response = MsgBox(Msg, Style, Title)  'if the table is already full
    End If
    InValEEPROM = I2CReadWord(I2C_EEPROM_Addr, 0)   'reading the first word
    If InValEEPROM = -1 Then
        Msg = "The EEPROM is not responding on I2C bus"  ' Define message.
        Style = vbOKOnly + vbCritical       ' Define buttons.
        Title = "Error"   ' Define title.
        Response = MsgBox(Msg, Style, Title)  'if the table is already full
    End If
    If InValPGA <> -1 And InValEEPROM <> -1 Then  'if both PGA309 and EEPROM
                                                  'are present
        Msg = "The PGA309 Bench is detected through I2C bus. "
        If InValEEPROM <> ROM_Programmed Then
            Msg = Msg & vbLf & "The EEPROM doesn't appear to be programmed for PGA309."
        End If
        Style = vbOKOnly + vbInformation    ' Define buttons.
        Title = "PGA309 Bench detected OK"   ' Define title.
        Response = MsgBox(Msg, Style, Title)  'if the table is already full
    End If
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub DelayClk(miliS As Long)
                ' argument is the delay in milisec
    Call Sleep(miliS)
End Sub
