Attribute VB_Name = "modulePortIO_Serial"
Option Explicit
Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
'Declare Sub Out Lib "INPOUT32.DLL" Alias "Out32" (ByVal Addr%, ByVal Value As Integer)
'Declare Function Inp Lib "INPOUT32.DLL" Alias "Inp32" (ByVal Addr%) As Integer

'declarations of constants for serial communication
' output register constants first
    Const SDA_ONE = False
    Const SDA_ZERO = True
    Const SCL_HIGH = False
    Const SCL_LOW = True

Public Const I2C_PGA309_Addr As Byte = &H80    ' 7'b1000000
Public Const I2C_EEPROM_Addr As Byte = &HA0    ' 7'b1010000
Public Const I2C_PortExp_Addr As Byte = &H70    ' PCF Port Expander Address: 7'b0111000
Public Const I2C_ADS1100_Addr As Byte = &H90    ' ADS1100 Address: 7'b1001000

    Const ReadBit As Byte = 1
    Const MaxIOAttempts = 5      'max number of attempts to communicate before bailout
    
Public Const ROM_Programmed As Long = &H5449    '0101_0100_0100_1001 -flag word of EEPROM
    ' EVM switch config lines on the PCF port expander
Public Const EVM_pga_testpin_high = &H80  ' EVM line controlling the PGA309 test pin
Public Const EVM_led2_off = &H40  ' EVM LED D2
Public Const EVM_led1_off = &H20  ' EVM LED D1
Public Const EVM_ads_cal = &H10  ' EVM ADS1100 input shorting switch for offset cal
Public Const EVM_i2c_join = &H4  ' EVM One-Wire enable line
Public Const EVM_Vs_dsbl = &H2  ' EVM Sensor board supply enable
Public Const EVM_OW_en = &H1  ' EVM One-Wire enable line

Public BoardConnected As Boolean
Public EVM_PortExp_Status As Byte
Public ADS1100Val As Long   'stored output of the ADS1100
Public ADS1100Offset As Long    ' stored calibration value
Public ADS1100Conf As Byte
Public OW_Vout_short As Boolean     ' variable that indicates that PRG is shorted to Vout
Public Testpin_high As Boolean     ' variable that indicates that Test pin was high in the last session
' Public BusClear As Integer     ' when =0 indicates I2C bus is clear for 1ms since last edge
Private CommCtrl As MSComm  ' local reference variable to the Comm control

Public Const MAX3238Timeout = 20000   'ms, 20sec interval to issue an activation on the interface
Public LastTransactionTime As Single ' stores the last time we user the port
                                     '- to see if 30sec had passed
'Public Ads1100_Error As Boolean

Public Sub InitPort()
    Dim InWord As Integer
 '   On Error Resume Next
            'initialize the port to set the I2C bus high
    Set CommCtrl = formMain.MSComm1
    CommCtrl.RTSEnable = SDA_ONE
    CommCtrl.DTREnable = SCL_HIGH
    Err.Clear
    On Error GoTo 0
End Sub

Public Function IsSomethingConnected()
'If CommCtrl <> 0 Then
    CommCtrl.RTSEnable = SDA_ONE
    CommCtrl.DTREnable = SCL_HIGH
    If CommCtrl.CTSHolding <> SDA_ONE Then
            ' the CTS line should be shorted to RTS: compare
        IsSomethingConnected = False    'return false - the bench is probably not connected
    Else    'next step: change  RTS state
        CommCtrl.RTSEnable = SDA_ZERO
        CommCtrl.DTREnable = SCL_HIGH
        If CommCtrl.CTSHolding = SDA_ZERO Then
                ' the CTS line should change its state as well
            IsSomethingConnected = True 'return true - the bench seems to be connected
        Else
            IsSomethingConnected = True 'return false - the bench is probably not connected
        End If
    End If
    Call I2CStartCond   'clear the I2C bus - just in case
    Call I2CStopCond
'End If
End Function

Public Sub StartMAX3238()
        ' if more than 20sec has elapsed since last port transaction
        ' issue start/stop, wait 10ms for the MAX3238 to start up, then proceed
    LastTransactionTime = timeGetTime
    Call I2CStartCond
    Call I2CStopCond
    DelayClk (10)
End Sub

Public Sub I2CStartCond()
  'Out Port_Data, (SDA_ONE Or SCL_HIGH)   'free I2C

    If (timeGetTime - LastTransactionTime) > MAX3238Timeout Then
            ' if more than 20sec has elapsed since last port transaction
            ' call MAX3238 to start up, then proceed
        Call StartMAX3238
    End If

   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_HIGH
  
  'Out Port_Data, (SDA_ZERO Or SCL_HIGH)
   CommCtrl.RTSEnable = SDA_ZERO
   CommCtrl.DTREnable = SCL_HIGH
  
  'Out Port_Data, (SDA_ZERO Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ZERO
   CommCtrl.DTREnable = SCL_LOW
  
End Sub
Public Sub I2CStopCond()
  'Out Port_Data, (SDA_ONE Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_LOW

  'Out Port_Data, (SDA_ZERO Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ZERO
   CommCtrl.DTREnable = SCL_LOW
  
  'Out Port_Data, (SDA_ZERO Or SCL_HIGH)
   CommCtrl.RTSEnable = SDA_ZERO
   CommCtrl.DTREnable = SCL_HIGH
  
' Out Port_Data, (SDA_ONE or SCL_HIGH)
' Out Port_Data, (SDA_one or SCL_low)
' Out Port_Data, (SDA_one or SCL_low)    'twice
  'Out Port_Data, (SDA_ONE Or SCL_HIGH)   'free I2C
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_HIGH
  
End Sub

Public Sub I2C_Master_Ack()
  'Out Port_Data, (SDA_ONE Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_LOW
  
  'Out Port_Data, (SDA_ZERO Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ZERO
   CommCtrl.DTREnable = SCL_LOW
  
  'Out Port_Data, (SDA_ZERO Or SCL_HIGH)
   CommCtrl.RTSEnable = SDA_ZERO
   CommCtrl.DTREnable = SCL_HIGH
  
  'Out Port_Data, (SDA_ZERO Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ZERO
   CommCtrl.DTREnable = SCL_LOW
  
End Sub
Public Sub I2C_No_Ack()
  'Out Port_Data, (SDA_ONE Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_LOW
  
  'Out Port_Data, (SDA_ONE Or SCL_HIGH)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_HIGH
  
  'Out Port_Data, (SDA_ONE Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_LOW
 
End Sub

Public Function I2CTransmitByte(SendByte As Byte) As Boolean
Dim j As Integer
Dim Bit As Boolean
  For j = 7 To 0 Step -1
    If (SendByte And CByte(2 ^ j)) <> 0 Then
        Bit = SDA_ONE 'if bit = 1
    Else
        Bit = SDA_ZERO
    End If
    'Out Port_Data, (Bit Or SCL_LOW)
    CommCtrl.RTSEnable = Bit
    CommCtrl.DTREnable = SCL_LOW

    'Out Port_Data, (Bit Or SCL_HIGH)
    CommCtrl.RTSEnable = Bit
    CommCtrl.DTREnable = SCL_HIGH

    'If (((Inp(Port_Status) \ 16) Xor 8) And 1) <> Bit Then
    If CommCtrl.DSRHolding <> Bit Then
            ' latch the SDA bit to the input and compare with the output
        I2CTransmitByte = False    'return false - the bench is probably not connected
        Exit Function   'bail out
    End If
   'Out Port_Data, (Bit Or SCL_LOW)
    CommCtrl.RTSEnable = Bit
    CommCtrl.DTREnable = SCL_LOW
    
  Next j
  'Out Port_Data, (SDA_ONE Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_LOW
  
  'Out Port_Data, (SDA_ONE Or SCL_HIGH)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_HIGH
  
  'Bit = ((Inp(Port_Status) \ 16) Xor 8) And 1   ' latch the acknowledgement bit
   Bit = CommCtrl.DSRHolding
  If Bit = SDA_ZERO Then   ' if SDA is low  - acknowlwdge recieved
    I2CTransmitByte = True
  Else                     ' if no acknowledgement
    I2CTransmitByte = False
  End If
End Function

Public Function I2CRecieveByte() As Byte
Dim j, BooleanToInt As Integer
  I2CRecieveByte = 0
  For j = 7 To 0 Step -1
    'Out Port_Data, (SDA_ONE Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_LOW
    
    'Out Port_Data, (SDA_ONE Or SCL_LOW)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_LOW
    
    'Out Port_Data, (SDA_ONE Or SCL_HIGH)
   CommCtrl.RTSEnable = SDA_ONE
   CommCtrl.DTREnable = SCL_HIGH
   
   'I2CRecieveByte = I2CRecieveByte + (2 ^ j) * (((Inp(Port_Status) \ 16) Xor 8) And 1)
   
   If CommCtrl.DSRHolding = False Then
    BooleanToInt = 1
   Else:
    BooleanToInt = 0
   End If
  I2CRecieveByte = I2CRecieveByte + (2 ^ j) * BooleanToInt
  Next j
End Function

Public Function I2CReadWord(DevAddr, Optional RegAddr As Byte = 255) As Long
        ' function reads a 16-bit (2-Byte) word from the I2C device
        ' register address must be 0 to 254
    Dim CtrlByte, InByte1, InByte2 As Byte
    Dim TryNum As Integer
    Dim Ackn As Boolean
            ' Reading the 2 bytes
    TryNum = 0
  If RegAddr <> 255 Then  ' if the register address set: writing the regaddr first
    Do
 '       DoEvents
        Call I2CStartCond
        Ackn = I2CTransmitByte(DevAddr And (Not ReadBit)) 'write the control byte to EEPROM
        If Ackn Then    'acknowledgement recieved
            Ackn = I2CTransmitByte(RegAddr)     'write the addressbyte to EEPROM
        End If
        TryNum = TryNum + 1
    Loop While (Not Ackn) And (TryNum < MaxIOAttempts)    'give it at least MaxIOAttempt tries
    If Not Ackn Then    'if the acknowledge was never obtained
        I2CReadWord = -1  'return -1 and bail out
        Exit Function
    End If
    Call I2CStopCond
  End If
    Call I2CStartCond
    Ackn = I2CTransmitByte(DevAddr Or ReadBit) 'write the control byte to EEPROM
    If Ackn Then    'acknowledgement recieved
        InByte1 = I2CRecieveByte()
        Call I2C_Master_Ack
        InByte2 = I2CRecieveByte()
        Call I2C_No_Ack
        Call I2CStopCond
        I2CReadWord = CLng(InByte2) * 256 + CLng(InByte1)
    Else    'if the acknowledge was not obtained
        I2CReadWord = -1  'return -1 and bail out
        Exit Function
    End If
End Function

Public Sub WritePGARegister(RegIndex As Integer)
    If TalkMode = 0 Then  'if the communication mode is I2C
        I2CWritePGARegister (RegIndex) ' read the register
                                'if no valid read - the function I2CWrite....
                                ' will display the message
    Else
            ' no cycling the power here - should be done one level above
        OW_WritePGARegister (RegIndex)
    End If
End Sub


Public Sub WritePGA()
                ' subroutine to write data into PGA registers
    Dim i As Integer
    Dim Success As Boolean
    Dim Keep_EVM_Status, Ackn
    formMain.MousePointer = 11 ' change pointer to hourglass
    formMain.StatusBar.SimpleText = "Writing PGA309 registers. Please stand by."
    If TalkMode = 1 And OW_Vout_short Then
            ' if PRG is shorted to Vout AND 1-wire mode:
            ' cycle the PGA309 power and hope that it will take
            ' less than 25ms to start transactions
        Call PGA_Power_Cycle
    End If
    For i = 1 To 7
        If TalkMode = 0 Then  'if the communication mode is I2C
                ' joining and splitting I2C bus is done inside I2CWritePGARegister
            Success = I2CWritePGARegister(i)
        Else        '1-wire write here
            Success = OW_WritePGARegister(i)
 
        End If
        If Not Success Then
                        'if no valid read
            Exit For
        End If
    Next i
    'NEW STUFF-- shuts off pull up after 1-wire talk apr 13, 05
     If TalkMode = 1 And OW_Vout_short Then
             Keep_EVM_Status = EVM_PortExp_Status
                                ' set P1='1'; P0='0'
             EVM_PortExp_Status = (EVM_PortExp_Status) And (254)   '!'
             Ackn = PCF_Write(False)     'execute w/o read back
               EVM_PortExp_Status = Keep_EVM_Status
    End If
    'END NEW STUFF apr 13, 05

    formMain.StatusBar.SimpleText = "Board Communcation Idle"
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub ReadPGARegister(RegIndex As Byte)
    Dim Inval As Long
    formMain.StatusBar.SimpleText = "Reading PGA309 register #" & CStr(RegIndex) & "..."
    If TalkMode = 0 Then  'if the communication mode is I2C
            ' issue a 1-wire transaction to shut-up PGA309
            ' (because of the bug in the port expander)
        OW_ReadPGARegister 0
                ' Join I2C buses             P2=1
        EVM_PortExp_Status = EVM_PortExp_Status Or EVM_i2c_join
        PCF_Write (False)

        Inval = I2CReadWord(I2C_PGA309_Addr, RegIndex) ' read the register
            ' split I2C buses             P2=1
        EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
        PCF_Write (False)
    Else        '1-wire read here
            ' no cycling the power here - should be done one level above
        Inval = OW_ReadPGARegister(CInt(RegIndex))
    End If
    If Inval = -1 Then    'if no valid read
        Dim Response
        Response = MsgBox("Error while reading the PGA309 register #" _
                & Str(RegIndex) & " Read aborted", _
                vbOKOnly + vbCritical, "Error")  'give user the message
        SetConnectedStatus (False)
    Else
        PGA309Registers(RegIndex) = Inval
        SetConnectedStatus (True)
        Call UpdatePhysDisplay(False)
    End If
    formMain.StatusBar.SimpleText = "Board Communcation Idle"
End Sub

Public Sub ReadPGA()
                ' subroutine to write data into PGA registers
    Dim i As Integer
    Dim Inval As Long
    Dim Keep_EVM_Status, Ackn
    formMain.MousePointer = vbHourglass ' change pointer to hourglass
    formMain.StatusBar.SimpleText = "Reading PGA309 registers. Please stand by."
    If TalkMode = 1 And OW_Vout_short Then
            ' if PRG is shorted to Vout AND 1-wire mode:
            ' cycle the PGA309 power and hope that it will take
            ' less than 25ms to start transactions
        '''Call PGA_Power_Cycle
            '!! note: this makes little sense, since we're going to read all zero's
            
            
        formMain.Toolbar1.Buttons.Item(12).Visible = True
        
                    
        'read EEPROM
        formMain.timerBusAlive.Enabled = False   'disable the timer
        Call ReadEEPROM
        formMain.tabRegRom.SelectedItem = formMain.tabRegRom.Tabs(1)
        Call formMain.chkKeepBusAlive_Click      'restore the timer
        'END READ EEPROM
        
        If LookupTableLength = 0 Then
            'reading the zero val
            ZERO_DAC_PRE_CAL = toTwosCompliment(LookupTable(0, 1))
            'RegBits2Index(1, 0, 15) = ZERO_DAC_PRE_CAL
            formMain.txtZDAC.Text = DisplayEngineering(VrefValue * ZERO_DAC_PRE_CAL / CDbl(65536))
            'reading the gain val
            Gain_DAC_PRE_CAL = toTwosCompliment(LookupTable(0, 2))
            'RegBits2Index(2, 0, 15) = Gain_DAC_PRE_CAL
            formMain.txtGDAC.Text = DisplayEngineering(GDACmin + ((GDACmax - GDACmin) * Gain_DAC_PRE_CAL / CDbl(65535)))
            Call formMain.txtZDAC_Validate(False)
            Call formMain.txtGDAC_Validate(False)
        End If
       Call formMain.cmdADCSingleShot_Click
        
        
        
         
        
        
        
    ElseIf TalkMode = 0 Then
        ' issue a 1-wire transaction to shut-up PGA309
            ' (because of the bug in the port expander)
        OW_ReadPGARegister 0
                ' Join I2C buses             P2=1
        EVM_PortExp_Status = EVM_PortExp_Status Or EVM_i2c_join
        PCF_Write (False)
    End If
    Inval = 0
If Not (OW_Vout_short) Or TalkMode = 0 Then ' SKIP WHEN ONE WIRE IS CONNECTED TO VOUT AND TALKING ON ONE WIRE ONLY
    For i = 0 To RegNumber - 1
        If TalkMode = 0 Then  'if the communication mode is I2C
            Inval = I2CReadWord(I2C_PGA309_Addr, CByte(i)) ' read the register
        Else        ' 1-wire read here
           Inval = OW_ReadPGARegister(i)
        End If
        If Inval = -1 Then    'if no valid read
            Dim Response
            Response = MsgBox("Error while reading the PGA309 register #" _
                & Str(i) & vbCrLf & " Read aborted", _
                vbOKOnly + vbCritical, "Error")  'give user the message
            SetConnectedStatus (False)
            Exit For
        Else
            PGA309Registers(i) = Inval
            SetConnectedStatus (True)
        End If
    Next i
End If ' END OF SKIP WHEN ONE WIRE IS CONNECTED TO VOUT AND TALKING ON ONE WIRE ONLY
    If TalkMode = 0 Then
                ' split I2C buses             P2=1
        EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
        PCF_Write (False)
    End If
    'NEW STUFF-- shuts off pull up after 1-wire talk apr 13, 05
    If TalkMode = 1 And OW_Vout_short Then
             Keep_EVM_Status = EVM_PortExp_Status
                                ' set P1='1'; P0='0'
             EVM_PortExp_Status = (EVM_PortExp_Status) And (254)   '!'
             Ackn = PCF_Write(False)     'execute w/o read back
               EVM_PortExp_Status = Keep_EVM_Status
    End If
    'END NEW STUFF apr 13, 05

    Call UpdatePhysDisplay(False)
    formMain.StatusBar.SimpleText = "Board Communcation Idle"
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub WriteEEPROM()
    If TalkMode = 0 Then  'if the communication mode is I2C
        Call I2CWriteEEPROM
    Else
        If OW_Vout_short Then
            ' if PRG is shorted to Vout AND 1-wire mode:
            ' cycle the PGA309 power and hope that it will take
            ' less than 25ms to start transactions
            Call PGA_Power_Cycle
        End If
        Call OW_WriteEEPROM
    End If
End Sub

Public Sub ReadEEPROM()
        'reads EEPROM content using either I2C or 1-wire routines
'    Dim InByte1, InByte2 As Byte
    Dim Ackn As Boolean
    Dim i, CurrAddr, EEPROMDataLength As Integer
    Dim InValue As Long
    Dim Msg, Style, Title, Response
    Dim Keep_EVM_Status
    formMain.MousePointer = vbHourglass ' change pointer to hourglass
    formMain.StatusBar.SimpleText = "Reading PGA309 config from EEPROM. Please stand by."
            ' Reading the 2 bytes
    If TalkMode = 0 Then    'Two-wire bus mode
            ' issue a 1-wire transaction to shut-up PGA309
            ' (because of the bug in the port expander)
        OW_ReadPGARegister 0
                ' Join I2C buses             P2=1
        EVM_PortExp_Status = EVM_PortExp_Status Or EVM_i2c_join
        Ackn = PCF_Write    ' (False)
        Call PGAShutUp      ' make sure I2C bus is free
        InValue = I2CReadWord(I2C_EEPROM_Addr, 0)   'reading the first word
                ' split I2C buses P2=0 again
        EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
        Ackn = PCF_Write    '(False)
    Else
        If OW_Vout_short Then
            ' if PRG is shorted to Vout AND 1-wire mode:
            ' cycle the PGA309 power and hope that it will take
            ' less than 25ms to start transactions
            Call PGA_Power_Cycle
        End If
        InValue = OW_ReadEEPROMWord(0)   '1-wire read
    End If
    If InValue = -1 Then
        Msg = "The EEPROM is not responding. Read aborted"  ' Define message.
        Style = vbOKOnly + vbCritical       ' Define buttons.
        Title = "Error"   ' Define title.
        Response = MsgBox(Msg, Style, Title)  'if the table is already full
        SetConnectedStatus (False)
        formMain.StatusBar.SimpleText = "Board Communcation Idle"
        formMain.MousePointer = 0
        Exit Sub
    ElseIf InValue <> ROM_Programmed Then
        Msg = "The EEPROM doesn't appear to be programmed for PGA309." & vbCrLf _
            & "Do you still want to load the contents into the memory?"   ' Define message.
        Style = vbYesNo + vbQuestion + vbDefaultButton2   ' Define buttons.
        Title = "Warning"   ' Define title.
        Response = MsgBox(Msg, Style, Title)
        If Response = vbNo Then
            SetConnectedStatus (True)
            formMain.StatusBar.SimpleText = "Board Communcation Idle"
            formMain.MousePointer = 0
            Exit Sub  ' if user chose No: exit
        ElseIf TalkMode = 1 And OW_Vout_short Then
            ' if PRG is shorted to Vout AND 1-wire mode:
            ' cycle the PGA309 power _AGAIN_
            Call PGA_Power_Cycle
        End If
    End If
    SetConnectedStatus (True)
    If TalkMode = 0 Then
            ' Join I2C buses P2=1
        EVM_PortExp_Status = EVM_PortExp_Status Or EVM_i2c_join
        Ackn = PCF_Write(False)
        Call PGAShutUp      ' make sure again I2C bus is free
    End If
    For i = 3 To 6      ' read the config registers
        If TalkMode = 0 Then
            PGA309Registers(i) = I2CReadWord(I2C_EEPROM_Addr, 2 * i) 'reading the word
        Else
            PGA309Registers(i) = OW_ReadEEPROMWord(2 * i)  '1-wire read
        End If
    Next i
    If TalkMode = 0 Then
        PGA309Registers(RegNumber) = I2CReadWord(I2C_EEPROM_Addr, 14)  'config checksum
    Else
        PGA309Registers(RegNumber) = OW_ReadEEPROMWord(14)   'config checksum
    End If
    LookupTableLength = -1
    For i = 0 To MaxLookupTableLength      ' read the lookuptable
        If TalkMode = 0 Then
            Call PGAShutUp      ' make sure again I2C bus is free
                 'reading the temp index
            LookupTable(i, 0) = fromTwosCompliment(I2CReadWord(I2C_EEPROM_Addr, 6 * i + 16))
                 'reading the zero val
            LookupTable(i, 1) = fromTwosCompliment(I2CReadWord(I2C_EEPROM_Addr, 6 * i + 18))
                 'reading the gain val
            LookupTable(i, 2) = fromTwosCompliment(I2CReadWord(I2C_EEPROM_Addr, 6 * i + 20))
        Else
                 'reading the temp index
            LookupTable(i, 0) = fromTwosCompliment(OW_ReadEEPROMWord(6 * i + 16))
                 'reading the zero val
            LookupTable(i, 1) = fromTwosCompliment(OW_ReadEEPROMWord(6 * i + 18))
                 'reading the gain val
            LookupTable(i, 2) = fromTwosCompliment(OW_ReadEEPROMWord(6 * i + 20))
        End If
        If LookupTable(i, 0) <> &H7FFF Then 'if it's not the end of the lookup table yet
            LookupTableLength = LookupTableLength + 1
        Else    'if we read the whole thing
            Exit For
        End If
    Next i
    If TalkMode = 0 Then    'Two-wire bus mode
                ' split I2C buses P2=0 again
        EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
        Ackn = PCF_Write(False)
    End If
    Call UpdateTableDisplay
    Call UpdatePhysDisplay(False)  'update the display w/o changing the
                                'checksum and wrting the registers back
                                
    'NEW STUFF-- shuts off pull up after 1-wire talk apr 13, 05
     If TalkMode = 1 And OW_Vout_short Then
             Keep_EVM_Status = EVM_PortExp_Status
                                ' set P1='1'; P0='0'
             EVM_PortExp_Status = (EVM_PortExp_Status) And (254)   '!'
             Ackn = PCF_Write(False)     'execute w/o read back
               EVM_PortExp_Status = Keep_EVM_Status
    End If
    'END NEW STUFF apr 13, 05

    formMain.StatusBar.SimpleText = "Board Communcation Idle"
    formMain.MousePointer = 0 ' change mouse back to default
        'update the status line here
End Sub

Public Sub I2CWriteEEPROM(Optional UseReg As Boolean = True, Optional InArray As Variant = Null)
        ' writes the EEPROM with either any bytes from the argument array or (if InArray is empty)
        ' generates the PGA309-compliant data set from the register and look-up table arrays
    Dim DataByte As Byte
    Dim Ackn As Boolean
    Dim i As Integer, CurrAddr As Integer, EEPROMDataLength As Integer, TryNum As Integer
    formMain.MousePointer = 11 ' change pointer to hourglass
    formMain.StatusBar.SimpleText = "Writing EEPROM through Two-Wire interface (directly). Please stand by."
                    ' create the data sequence for EEPROM
    If UseReg Then
                    'in the input is empty - create the data stream
        EEPROMDataLength = CreateDataStream()
    Else
        EEPROMDataLength = UBound(InArray)
    End If
        ' issue a 1-wire transaction to shut-up PGA309
        ' (because of the bug in the port expander)
    OW_ReadPGARegister 0
        ' Join I2C buses P2=1
    EVM_PortExp_Status = EVM_PortExp_Status Or EVM_i2c_join
    Ackn = PCF_Write(False)
        'now start the writing process 8 bytes at a time
    CurrAddr = 0
    TryNum = 0
    Do
        Call PGAShutUp      ' make sure I2C bus is free
        DataByte = I2C_EEPROM_Addr And (Not ReadBit)  'WRITE control word
        Call I2CStartCond
        Ackn = I2CTransmitByte(DataByte) 'write the control byte to EEPROM
        If Ackn Then    'acknowledgement recieved
            DataByte = CurrAddr    'setting the beginning address
            Ackn = I2CTransmitByte(DataByte) 'write the address byte to EEPROM
            If Ackn Then    'acknowledgement recieved
                Do While (i <= (CurrAddr + 7)) And (i <= EEPROMDataLength)
                                                    'but it should be OK
                    If UseReg Then
                        DataByte = EEPROMDataStream(i)    'next byte in sequence
                    Else
                        DataByte = InArray(i)
                    End If
                    Ackn = I2CTransmitByte(DataByte) 'write the data byte to EEPROM
                    If Not Ackn Then Exit Do
                    i = i + 1
                Loop
            End If
        End If
        Call I2CStopCond
        If Ackn Then    'if all the acknowledgements are OK - generate stop
                        ' otherwise repeat the sequience w/o increasing
                        ' the address counter
            CurrAddr = CurrAddr + 8 '!!0
            TryNum = 0
        Else
            TryNum = TryNum + 1
        End If
    Loop While ((Not Ackn) Or (CurrAddr <= EEPROMDataLength)) And (TryNum < MaxIOAttempts)
    If Not Ackn Then    ' if the acknowledge was not recieved after MaxIOAttempts
            MsgBox "The EEPROM is not responding. Write aborted", _
                vbOKOnly + vbCritical, "Error"  'give user the message
            SetConnectedStatus (False)
    Else
        SetConnectedStatus (True)
    End If
        ' Split I2C buses P2=0
    EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
    Ackn = PCF_Write(False)
    formMain.StatusBar.SimpleText = "Board Communcation Idle"
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Function I2CWritePGARegister(RegIndex As Integer) As Boolean
                ' subroutine to write the value into one PGA309 register
                ' returns "True" for success ans "False" for failure to write
    Dim DataByte As Byte
    Dim Ackn As Boolean
    Dim i, TryNum As Integer
    Dim Reply As Variant
    formMain.MousePointer = 11 ' change pointer to hourglass
    formMain.StatusBar.SimpleText = "Writing PGA309 register #" & CStr(RegIndex) & " through Two-Wire. Please stand by."
            ' issue a 1-wire transaction to shut-up PGA309
            ' (because of the bug in the port expander)
    OW_ReadPGARegister 0
            ' Join I2C buses             P2=1
    EVM_PortExp_Status = EVM_PortExp_Status Or EVM_i2c_join
    PCF_Write (False)
    TryNum = 0
    Do
        DataByte = I2C_PGA309_Addr And (Not ReadBit) 'WRITE control word
        Call I2CStartCond
        Ackn = I2CTransmitByte(DataByte) 'write the control byte to PGA309
        If Ackn Then    'acknowledgement recieved
            DataByte = RegIndex    'setting the beginning address
            Ackn = I2CTransmitByte(DataByte) 'write the address byte to PGA309
            If Ackn Then    'acknowledgement recieved
                DataByte = CByte(PGA309Registers(RegIndex) And &HFF)        'LSB
                Ackn = I2CTransmitByte(DataByte) 'write the LSB byte to PGA
                If Ackn Then
                    DataByte = CByte((PGA309Registers(RegIndex) And &HFF00) \ 256)    'MSB
                    Ackn = I2CTransmitByte(DataByte) 'write the MSB byte to PGA
                End If
            End If
        End If
        If Ackn Then    'if all the acknowledgements are OK - generate stop
                        ' otherwise repeat the sequience w/o increasing
                        ' the address counter
            Call I2CStopCond
        Else
            TryNum = TryNum + 1
        End If
    Loop While ((Not Ackn) And (TryNum < MaxIOAttempts))
        ' split I2C buses             P2=0
    EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
    PCF_Write (False)
    If Not Ackn Then    ' if the acknowledge was not recieved after MaxIOAttempts
            Dim Response
            Response = MsgBox("The PGA309 is not responding while writing register #" _
                & Str(RegIndex) & vbCrLf & ". Write aborted", _
                vbOKOnly + vbCritical, "Error")  'give user the message
    End If
    I2CWritePGARegister = Ackn
    formMain.StatusBar.SimpleText = "Board Communcation Idle"
    formMain.MousePointer = 0 ' change mouse back to default
End Function

Public Function PCF_Write(Optional Verify As Boolean = True) As Boolean
        ' the function updates the PCF8574A with the current value of
        ' EVM_PortExp_Status
    Dim TmpvalPortExp As Byte
    Dim Ackn As Boolean
'    Call PGAShutUp      ' make sure I2C bus is free
    TmpvalPortExp = EVM_PortExp_Status  ' temporary storage
    Call I2CStartCond
    Ackn = I2CTransmitByte(I2C_PortExp_Addr And (Not ReadBit)) 'address PCF8574 for Writing
    If Ackn Then
        Ackn = I2CTransmitByte(EVM_PortExp_Status) 'write the status in
        Call I2CStopCond
        If Ackn And Verify Then
            Ackn = PCF_read()   ' read back the value from the PCF8574
            If TmpvalPortExp <> EVM_PortExp_Status Then
                    ' if read-back value is not the same as the written - ERROR
                Ackn = False
'                EVM_PortExp_Status = TmpvalPortExp  ' restore the value in memory
            End If
        End If
    Else
        Call I2CStopCond        ' free up the I2C bus
    End If
    If ADS1100.Visible Then             ' update the display
        Call ADS1100.ShowPCFStatus
    End If
    PCF_Write = Ackn
End Function

Public Function PCF_read() As Boolean
    Dim Ackn As Boolean
    Dim Inval As Byte
'    Call PGAShutUp      ' make sure I2C bus is free
    Call I2CStartCond
    Ackn = I2CTransmitByte(I2C_PortExp_Addr Or ReadBit) 'write the READ control byte to PCF8574
    If Ackn Then    'if the acknowledge was obtained
        Inval = I2CRecieveByte()
        Call I2C_No_Ack
        EVM_PortExp_Status = Inval
        If ADS1100.Visible Then
            Call ADS1100.ShowPCFStatus
        End If
    End If
    Call I2CStopCond
    PCF_read = Ackn
End Function

Public Sub DetectEVMBoard()
        ' The procedure detects the I2C port expander on the EVM, enables the EVM
        ' power and lights up the diode D1
    Dim Ackn, ReadCont As Boolean
    Dim Msg, Style, Title, Response
    formMain.MousePointer = 11 ' change pointer to hourglass
    formMain.StatusBar.SimpleText = "Detecting PGA309 board. Please stand by."
    ReadCont = False    ' do not load the EEPROM contents by default
    Ackn = IsSomethingConnected
    If Ackn Then   ' if the proper lines are connected on the serial port (board present)
           ' Reading 1 byte from the port expander
        Ackn = PCF_read ' read the PCF8574
        If Ackn Then    'if the acknowledge was obtained
            If Not BoardConnected Then
                    'if the board hadn't been previously detected
                Msg = "The PC Interface Board appears to be connected"
                If (EVM_PortExp_Status And &H2) <> 0 Then   'if PGA309 is not powered
                    Msg = Msg & " but PGA309 is not yet powered." & vbCrLf _
                     & "The Sensor interface Board will be powered up whether Yes or No is chosen"
                End If
                Msg = Msg & "." & vbCrLf & _
                     "Do you want to load the EEPROM and register contents into the PC memory?"
                Style = vbYesNo + vbQuestion + vbDefaultButton1   ' Define buttons.
                Title = "PGA309 Designer's Kit"   ' Define title.
                Response = MsgBox(Msg, Style, Title)
                If Response = vbYes Then
                    ReadCont = True
                End If
            End If
                ' Start configure the PCF8574A to something resonable
                        ' enbl PGA309 power             P1=0
            EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_Vs_dsbl)
                        ' split I2C buses             P2=0
            EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
                        ' connect the ADS1100 to the PGA309: P4=0
            EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_ads_cal)
                        ' NOTE: 17.10.03 - made always P0=1 - doesn't hurt in I2C mode either
            EVM_PortExp_Status = EVM_PortExp_Status Or EVM_OW_en
                        ' LED1 indicates that the board is talking to the PC
                        ' set the diode D1 to light up: P5=0
            EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_led1_off)
            If Testpin_high Then     ' if Test pin was set high in the last session
                        ' set P7 and turn ON LED2 -
                        ' the "_click" procedure in FormMain will take care of setting the bits
                formMain.chkTestPinHigh.Value = 1
            Else        'clear P7 and turn OFF LED2
                formMain.chkTestPinHigh.Value = 0
            End If
            Ackn = PCF_Write()
            If Ackn Then    ' if EVM config is OK - detect PGA309
                If Not ReadCont Then    'just detect the bench - no loading the EEPROM
                    If TalkMode = 0 Then
                        Call I2CDetectBench
                    Else
                        If OW_Vout_short Then
                            ' if PRG is shorted to Vout AND 1-wire mode:
                            ' cycle the PGA309 power and hope that it will take
                            ' less than 25ms to start transactions
                            Call PGA_Power_Cycle
                        End If
                        Call OW_DetectBench
                    End If
                    'If Testpin_high Then                            'added sep 28 04 AAK
                    '    formMain.timerBusAlive.Enabled = False      'disable the timer
                   '     Call ReadPGA
                   '     Call formMain.chkKeepBusAlive_Click         'restore the timer
                   ' End If                                          'added sep 28 04 AAK
                         'read the PGA309 output
                Else    'load the registers and EEPROM
    
                    Call ReadPGA
                    If BoardConnected Then  ' if there was no error reading the PGA309
                        Call ReadEEPROM
                   End If
                End If
                Call ADS1100_read
            End If
        End If
    End If
    If Not Ackn Then    'acknowledgement NOT received
        Call I2CStopCond
        Msg = "The PGA309 Board is not responding"  ' Define message for the case of error.
        Style = vbOKOnly + vbCritical       ' Define buttons.
        Title = "Error"   ' Define title.
        Response = MsgBox(Msg, Style, Title)    ' give an error message!
        SetConnectedStatus (False)
    End If
    formMain.StatusBar.SimpleText = "Board Communcation Idle"
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub I2CDetectBench()
            ' the routine checks for the acknowledgements and reads from PGA309 and EEPROM,
    Dim InValPGA, InValEEPROM As Long
    Dim Msg, Style, Title, Response
    formMain.MousePointer = 11 ' change pointer to hourglass
            ' Reading the 2 bytes from the PGA309
            ' issue a 1-wire transaction to shut-up PGA309
            ' (because of the bug in the port expander)
    OW_ReadPGARegister 0
            ' Join I2C buses             P2=1
    EVM_PortExp_Status = EVM_PortExp_Status Or EVM_i2c_join
    PCF_Write (False)
    InValPGA = I2CReadWord(I2C_PGA309_Addr, 0)   'reading the first word
    If InValPGA = -1 Then
        Msg = "The PGA309 is not responding on Two-Wire bus"  ' Define message.
        Style = vbOKOnly + vbCritical       ' Define buttons.
        Title = "Error"   ' Define title.
        Response = MsgBox(Msg, Style, Title)
        SetConnectedStatus (False)
    End If
    InValEEPROM = I2CReadWord(I2C_EEPROM_Addr, 0)   'reading the first word
        ' split I2C buses P2=0
    EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
    PCF_Write (False)
    If InValEEPROM = -1 Then
        Msg = "The EEPROM is not responding on Two-Wire bus"  ' Define message.
        Style = vbOKOnly + vbCritical       ' Define buttons.
        Title = "Error"   ' Define title.
        Response = MsgBox(Msg, Style, Title)
        SetConnectedStatus (False)
    End If
    If InValPGA <> -1 And InValEEPROM <> -1 Then  'if both PGA309 and EEPROM
                                                  'are present
        Msg = "The PC interface board and PGA309 are detected through Two-Wire bus."
        If InValEEPROM <> ROM_Programmed Then
            Msg = Msg & vbLf & "The EEPROM doesn't appear to be programmed for PGA309."
        End If
        Style = vbOKOnly + vbInformation    ' Define buttons.
        Title = "PGA309 Designer's Kit"   ' Define title.
        Response = MsgBox(Msg, Style, Title)  'if the table is already full
        SetConnectedStatus (True)
    End If
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub I2CGenCallReset()
                ' subroutine to write the value into one PGA309 register
                ' returns "True" for success ans "False" for failure to write
    Dim Ackn As Boolean
    formMain.MousePointer = 11 ' change pointer to hourglass
    Call I2CStartCond
    Ackn = I2CTransmitByte(&H0)  'write the zero address byte
    If Ackn Then    'acknowledgement recieved
        Ackn = I2CTransmitByte(&H6)  'write the reset command
    End If
    Call I2CStopCond        'generate stop
    If Not Ackn Then    ' if the acknowledge was not recieved
            Dim Response
            Response = MsgBox("No devices acknowledged the Two-Wire General Call Reset", _
                vbOKOnly + vbCritical, "Error")  'give user the message
    End If
    formMain.MousePointer = 0 ' change mouse back to default
End Sub

Public Sub ADS1100_read()
        'reads the ADS1100 output and config registers
        'and updates the dialog boxes
    Dim InByte1, InByte2 As Byte
    Dim Ackn As Boolean
    formMain.StatusBar.SimpleText = "Updating Vout reading"
    formMain.timerBusAlive.Enabled = False   'disable the timer
'    Call PGAShutUp      ' make sure I2C bus is free
    Call I2CStartCond
    Ackn = I2CTransmitByte(I2C_ADS1100_Addr Or ReadBit) 'write the control byte to EEPROM
    If Ackn Then    'acknowledgement recieved
        InByte1 = I2CRecieveByte()
        Call I2C_Master_Ack
        InByte2 = I2CRecieveByte()
        Call I2C_Master_Ack
        ADS1100Conf = I2CRecieveByte()
        Call I2C_No_Ack
        Call I2CStopCond
        ADS1100Val = CLng(InByte1) * 256 + CLng(InByte2)
       ' Ads1100_Error = False
    Else    'if the acknowledge was not obtained
        Dim Response
        Response = MsgBox("The ADS1100 is not responding. Read aborted", _
                vbOKOnly + vbCritical, "Error")  'give user the message
        If ADS1100.chkADS1100ContRead.Value = 1 Then
            ADS1100.chkADS1100ContRead.Value = 0  ' disable the continous read time just in case
        End If
      '  Ads1100_Error = True
    End If
    Call formMain.chkKeepBusAlive_Click      'restore the timer
    Call ADS1100_display
    formMain.StatusBar.SimpleText = "Board Communcation Idle"
End Sub
    
Public Sub ADS1100_display()
    Dim ADSgain As Single
    Dim ADSresln As Integer
    Dim ADSVoltVal As Double
    ADSgain = 1#
    ADSresln = 16
    If (ADS1100Conf And 1) Then     ' bit0 = '1'
        ADSgain = ADSgain * 2
    End If
    If (ADS1100Conf And 2) Then     ' bit 1 = '1'
        ADSgain = ADSgain * 2
    End If
    If (ADS1100Conf And 8) <> 0 And (ADS1100Conf And 4) <> 0 Then ' bit 3,2 = '11'
        ADSresln = 16
    ElseIf (ADS1100Conf And 8) <> 0 And (ADS1100Conf And 4) = 0 Then ' bit 3,2 = '10'
        ADSresln = 15
    ElseIf (ADS1100Conf And 8) = 0 And (ADS1100Conf And 4) <> 0 Then ' bit 3,2 = '01'
        ADSresln = 14
    Else                    ' bit 3,2 = '00'
        ADSresln = 12
    End If
    ADSVoltVal = (fromTwosCompliment(ADS1100Val) - fromTwosCompliment(ADS1100Offset)) _
        / ADSgain * EngineeringVal(formMain.txtEVMVs.Text) / (2# ^ (ADSresln - 1))
            'volt value
    formMain.txtADS1100(2).Text = Hex(ADS1100Val)   'hex
    formMain.txtADS1100(1).Text = CStr(fromTwosCompliment(ADS1100Val))  'decimal
    formMain.txtADS1100(0).Text = Format(CStr(ADSVoltVal), "0.00000") 'volts
    If ADS1100.Visible Then
            'if the ADS1100 dialog box is visible
        ADS1100.txtADSout(2).Text = formMain.txtADS1100(2).Text
        ADS1100.txtADSout(1).Text = formMain.txtADS1100(1).Text
        ADS1100.txtADSout(0).Text = formMain.txtADS1100(0).Text
        Call ADS1100.ADC_Out_reg
        Call ADS1100.ShowADS1100ConfReg
        Call ADS1100.ADCbitColor
    End If
End Sub

Public Sub PGA_Power_Cycle()
                    ' power off the PGA309 and the 1Wire-interface
    Dim Ackn As Boolean
    Dim Keep_EVM_Status As Byte
    Keep_EVM_Status = EVM_PortExp_Status
                                ' set P1='1'; P0='0'
    EVM_PortExp_Status = (EVM_PortExp_Status Or EVM_Vs_dsbl) And (Not EVM_OW_en)  '!'
    Ackn = PCF_Write(False)     'execute w/o read back
    EVM_PortExp_Status = Keep_EVM_Status   'restore the port status
    If Ackn Then
        Delay (100)     'wait for all bypass caps to dischange
        Ackn = PCF_Write(False) ' execute restore
        DelayClk (2)    'wait for power to settle - hard wait, no events!
    End If
End Sub

Public Sub DelayClk(miliS As Long)
                ' argument is the delay in milisec - hard wait,
                ' no events are allowed during wait period
    Call Sleep(miliS)
End Sub

Public Sub SetConnectedStatus(Optional Status As Boolean = False)
            ' show the "connected" picture on the main form
    If Status Then
        formMain.picConnected.Picture = formMain.ImageListBig.ListImages(1).Picture
        formMain.picConnected.ToolTipText = "The board is Detected OK!"
        BoardConnected = True
    Else
        formMain.picConnected.Picture = formMain.ImageListBig.ListImages(2).Picture
        formMain.picConnected.ToolTipText = "The board is NOT Detected!"
        BoardConnected = False
    End If
End Sub

Public Sub PGAShutUp()
    ' generates  a sequense to cause the PGA309 to stop I2C communications
    ' until the host is done (reading from  ADS1100 or from the EEPROM, etc)
    Static NoMoreWarn1 As Boolean, NoMoreWarn2 As Boolean
    Static LastShutUp As Single ' stores the last time we issued the message
    Dim TempADCval As Long
    Const Tinterval = 750   'ms, interval to repeat the ShutUp
    If Testpin_high Or (timeGetTime - LastShutUp) < Tinterval Then
            ' if the Test pin is already high or
            ' less than 0.75s has elapsed since last time - do nothing
        Exit Sub
    Else    'reset the interval
        LastShutUp = timeGetTime
    End If
    
' removed the first "if talkmode = 1" - want to issue 1-wire transaction anytime Vout and
'   PRG are not shorted - just in case - it is the easiest way to shut PGA309 up.
'**    If TalkMode = 1 Then     ' 1-wire mode
        If Not OW_Vout_short Then
            TempADCval = OW_ReadPGARegister(0)
'        ElseIf TalkMode = 1 And Not NoMoreWarn1 Then  ' warn once that we can't free up I2C bus when
'                                    ' the PRG and Vout are shorted together
'            MsgBox "Debug Mesage:" & vbCrLf & _
'                "Two-Wire bus freeing sequence will be used when PRG and Vout are shorted." & vbCrLf & _
'                vbCrLf & "This message will only appear once per session.", vbOKOnly + vbExclamation, "PGA309 Designer's Kit"
'            NoMoreWarn1 = True
        End If
 '**   End If
    If TalkMode = 0 Or (TalkMode = 1 And OW_Vout_short) Then    'I2C mode or 1-wire with Vout short
        Dim DataByte As Byte
        Dim Ackn As Boolean
        Dim Try As Integer
        Try = 0
        DataByte = I2C_PGA309_Addr And (Not ReadBit) 'WRITE control word
        Do
            Call I2CStartCond
            Ackn = I2CTransmitByte(DataByte) 'write the control byte to PGA309
            Call I2CStopCond
            ' trying Stop condition - will PGA309 wait 1s afterwards? YES, but not always!
            ' adding 10 attempts
            Try = Try + 1
        Loop Until Ackn Or Try >= 10
        If Not Ackn Then
            If Not NoMoreWarn2 And BoardConnected Then  ' if no ackn and board connected -  warn the user
                MsgBox "Debug Mesage:" & vbCrLf & _
                "PGA309 didn't acknowledge the Shut Up message." & vbCrLf & _
                "This may happen when SDA and SCL from the Sensor board " & vbCrLf & _
                "are not connected to the PC interface board." & vbCrLf & _
                "Otherwise you may see errors reading EEPROM" & vbCrLf & _
                "To prevent, do one of the following:" & vbCrLf & _
                "1. Pull Test pin on PGA309 high to disable EEPROM readback; OR" & vbCrLf & _
                "2. Use One-Wire interface to communicate to PGA309." & vbCrLf & _
                vbCrLf & "This message will only appear once per session.", vbOKOnly + vbExclamation, "PGA309 Designer's Kit"
                NoMoreWarn2 = True
            End If
        End If
    End If
End Sub

Public Function ReadFull1KEEPROM() As Byte()
        'reads EEPROM content using either I2C or 1-wire routines
    Dim ReturnArray() As Byte
    Dim Ackn As Boolean
    Dim i As Byte, CurrAddr As Integer, EEPROMDataLength As Integer
    Dim InValue As Long
    Dim Msg, Style, Title, Response
        'put status line indicator here
    formMain.MousePointer = vbHourglass ' change pointer to hourglass
    formMain.StatusBar.SimpleText = "Reading Full 1kBit EEPROM content. Please stand by."
    If TalkMode = 1 And OW_Vout_short Then
        ' if PRG is shorted to Vout AND 1-wire mode:
        ' cycle the PGA309 power and hope that it will take
        ' less than 25ms to start transactions
        Call PGA_Power_Cycle
    ElseIf TalkMode = 0 Then   'if I2C mode
            ' issue a 1-wire transaction to shut-up PGA309
            ' (because of the bug in the port expander)
        OW_ReadPGARegister 0
                ' Join I2C buses             P2=1
        EVM_PortExp_Status = EVM_PortExp_Status Or EVM_i2c_join
        Ackn = PCF_Write(False)
    End If
    ReDim ReturnArray(127)
    For i = 0 To 127 Step 2
            ' Reading 2 bytes
        If TalkMode = 0 Then
            Call PGAShutUp      ' make sure I2C bus is free
            InValue = I2CReadWord(I2C_EEPROM_Addr, i) 'reading the word
        Else
            InValue = OW_ReadEEPROMWord(i) '1-wire read
        End If
        If InValue = -1 Then
            Msg = "The EEPROM is not responding. Read aborted"  ' Define message.
            Style = vbOKOnly + vbCritical       ' Define buttons.
            Title = "Error"   ' Define title.
            Response = MsgBox(Msg, Style, Title)  'if the table is already full
            SetConnectedStatus (False)
            Exit For
        Else
            ReturnArray(i) = CByte(InValue Mod 256)
            ReturnArray(i + 1) = CByte(InValue \ 256)
        End If
    Next i
    If TalkMode = 0 Then   'if I2C mode
                ' Split I2C buses  P2=0
        EVM_PortExp_Status = EVM_PortExp_Status And (Not EVM_i2c_join)
        Ackn = PCF_Write(False)
    End If
    formMain.StatusBar.SimpleText = "Board Communcation Idle"
    formMain.MousePointer = 0
        'put status line indicator here
    If i > 0 Then
        'if we have at least one value to return
        ReDim Preserve ReturnArray(i - 1) 'trim the array size
        ReadFull1KEEPROM = ReturnArray
    Else
        ReadFull1KEEPROM = ""
    End If
End Function

Public Sub WriteFull1KEEPROM(InArray() As Byte)
    If TalkMode = 0 Then  'if the communication mode is I2C
        Call I2CWriteEEPROM(False, InArray)
    Else    ' 1-wire interface
        If OW_Vout_short Then
            ' if PRG is shorted to Vout AND 1-wire mode:
            ' cycle the PGA309 power and hope that it will take
            ' less than 25ms to start transactions
            Call PGA_Power_Cycle
        End If
        Call OW_WriteEEPROM(False, InArray)
    End If
End Sub

